/**
 * A specialized set of basic functions utilities that are highly reusable without creating any dependencies : arrays, strings, chars, objects, numbers, maths, date, colors, etc. - version: 1.0.5 - license: MPL 2.0/GPL 2.0+/LGPL 2.1+ - Follow me on Twitter! @ekameleon
 */

(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    (factory());
}(this, (function () { 'use strict';

    var object;
    try {
      object = global || null;
    } catch (ignore) {}
    if (!object) {
      try {
        object = window;
      } catch (e) {}
    }
    if (!object) {
      try {
        object = document;
      } catch (ignore) {}
    }
    if (!object) {
      object = {};
    }
    var global = object;

    var skip = false;
    function sayHello() {
      var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
      var version = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      var link = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
      if (skip) {
        return;
      }
      try {
        if (navigator && navigator.userAgent && navigator.userAgent.toLowerCase().indexOf('chrome') > -1) {
          var args = ["\n %c %c %c ".concat(name, " ").concat(version, " %c %c ").concat(link, " %c %c\n\n"), 'background: #ff0000; padding:5px 0;', 'background: #AA0000; padding:5px 0;', 'color: #F7FF3C; background: #000000; padding:5px 0;', 'background: #AA0000; padding:5px 0;', 'color: #F7FF3C; background: #ff0000; padding:5px 0;', 'background: #AA0000; padding:5px 0;', 'background: #ff0000; padding:5px 0;'];
          window.console.log.apply(console, args);
        } else if (window.console) {
          window.console.log("".concat(name, " ").concat(version, " - ").concat(link));
        }
      } catch (error) {
      }
    }
    function skipHello() {
      skip = true;
    }

    function toUnicodeNotation(num) {
      var hex = num.toString(16);
      while (hex.length < 4) {
        hex = "0" + hex;
      }
      return hex;
    }

    function dump(value) {
      var prettyprint = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var indent = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var indentor = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : "    ";
      indent = isNaN(indent) ? 0 : indent;
      prettyprint = Boolean(prettyprint);
      if (!indentor) {
        indentor = "    ";
      }
      if (value === undefined) {
        return "undefined";
      } else if (value === null) {
        return "null";
      } else if (typeof value === "string" || value instanceof String) {
        return dumpString(value);
      } else if (typeof value === "boolean" || value instanceof Boolean) {
        return value ? "true" : "false";
      } else if (typeof value === "number" || value instanceof Number) {
        return value.toString();
      } else if (value instanceof Date) {
        return dumpDate(value);
      } else if (value instanceof Array) {
        return dumpArray(value, prettyprint, indent, indentor);
      } else if (value.constructor && value.constructor === Object) {
        return dumpObject(value, prettyprint, indent, indentor);
      } else if ("toSource" in value) {
        return value.toSource(indent);
      } else {
        return '<unknown>';
      }
    }
    function dumpArray(value) {
      var prettyprint = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var indent = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var indentor = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : "    ";
      indent = isNaN(indent) ? 0 : indent;
      prettyprint = Boolean(prettyprint);
      if (!indentor) {
        indentor = "    ";
      }
      var source = [];
      var i;
      var l = value.length;
      for (i = 0; i < l; i++) {
        if (value[i] === undefined) {
          source.push("undefined");
          continue;
        }
        if (value[i] === null) {
          source.push("null");
          continue;
        }
        if (prettyprint) {
          indent++;
        }
        source.push(dump(value[i], prettyprint, indent, indentor));
        if (prettyprint) {
          indent--;
        }
      }
      if (prettyprint) {
        var spaces = [];
        for (i = 0; i < indent; i++) {
          spaces.push(indentor);
        }
        var decal = '\n' + spaces.join('');
        return decal + "[" + decal + indentor + source.join("," + decal + indentor) + decal + "]";
      } else {
        return "[" + source.join(",") + "]";
      }
    }
    function dumpDate(date
    ) {
      var timestamp = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      timestamp = Boolean(timestamp);
      if (timestamp) {
        return "new Date(" + String(date.valueOf()) + ")";
      } else {
        var y = date.getFullYear();
        var m = date.getMonth();
        var d = date.getDate();
        var h = date.getHours();
        var mn = date.getMinutes();
        var s = date.getSeconds();
        var ms = date.getMilliseconds();
        var data = [y, m, d, h, mn, s, ms];
        data.reverse();
        while (data[0] === 0) {
          data.splice(0, 1);
        }
        data.reverse();
        return "new Date(" + data.join(",") + ")";
      }
    }
    function dumpObject(value) {
      var prettyprint = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var indent = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var indentor = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : "    ";
      indent = isNaN(indent) ? 0 : indent;
      prettyprint = Boolean(prettyprint);
      if (!indentor) {
        indentor = "    ";
      }
      var source = [];
      for (var member in value) {
        if (value.hasOwnProperty(member)) {
          if (value[member] === undefined) {
            source.push(member + ":" + "undefined");
            continue;
          }
          if (value[member] === null) {
            source.push(member + ":" + "null");
            continue;
          }
          if (prettyprint) {
            indent++;
          }
          source.push(member + ":" + dump(value[member], prettyprint, indent, indentor));
          if (prettyprint) {
            indent--;
          }
        }
      }
      source = source.sort();
      if (prettyprint) {
        var spaces = [];
        for (var i = 0; i < indent; i++) {
          spaces.push(indentor);
        }
        var decal = '\n' + spaces.join('');
        return decal + '{' + decal + indentor + source.join(',' + decal + indentor) + decal + '}';
      } else {
        return '{' + source.join(',') + '}';
      }
    }
    function dumpString(value) {
      var code;
      var quote = "\"";
      var str = "";
      var ch = "";
      var pos = 0;
      var len = value.length;
      while (pos < len) {
        ch = value.charAt(pos);
        code = value.charCodeAt(pos);
        if (code > 0xFF) {
          str += "\\u" + toUnicodeNotation(code);
          pos++;
          continue;
        }
        switch (ch) {
          case "\b":
            {
              str += "\\b";
              break;
            }
          case "\t":
            {
              str += "\\t";
              break;
            }
          case "\n":
            {
              str += "\\n";
              break;
            }
          case "\x0B":
            {
              str += "\\v";
              break;
            }
          case "\f":
            {
              str += "\\f";
              break;
            }
          case "\r":
            {
              str += "\\r";
              break;
            }
          case "\"":
            {
              str += "\\\"";
              break;
            }
          case "'":
            {
              str += "\\\'";
              break;
            }
          case "\\":
            {
              str += "\\\\";
              break;
            }
          default:
            {
              str += ch;
            }
        }
        pos++;
      }
      return quote + str + quote;
    }

    function trace(context) {
      if (console) {
        console.log(context);
      }
    }

    var performance = global.performance || {};
    Object.defineProperty(global, 'performance', {
      value: performance,
      configurable: true,
      writable: true
    });
    performance.now = performance.now || performance.mozNow || performance.msNow || performance.oNow || performance.webkitNow;
    if (!(global.performance && global.performance.now)) {
      var startTime = Date.now();
      global.performance.now = function () {
        return Date.now() - startTime;
      };
    }

    var ONE_FRAME_TIME = 16;
    var lastTime = Date.now();
    var vendors = ['ms', 'moz', 'webkit', 'o'];
    var len = vendors.length;
    for (var x = 0; x < len && !global.requestAnimationFrame; ++x) {
      var p = vendors[x];
      global.requestAnimationFrame = global["".concat(p, "RequestAnimationFrame")];
      global.cancelAnimationFrame = global["".concat(p, "CancelAnimationFrame")] || global["".concat(p, "CancelRequestAnimationFrame")];
    }
    if (!global.requestAnimationFrame) {
      global.requestAnimationFrame = function (callback) {
        if (typeof callback !== 'function') {
          throw new TypeError("".concat(callback, "is not a function"));
        }
        var currentTime = Date.now();
        var delay = ONE_FRAME_TIME + lastTime - currentTime;
        if (delay < 0) {
          delay = 0;
        }
        lastTime = currentTime;
        return setTimeout(function () {
          lastTime = Date.now();
          callback(performance.now());
        }, delay);
      };
    }
    if (!global.cancelAnimationFrame) {
      global.cancelAnimationFrame = function (id) {
        return clearTimeout(id);
      };
    }
    var cancelAnimationFrame = global.cancelAnimationFrame;
    var requestAnimationFrame = global.requestAnimationFrame;

    var isBoolean = function isBoolean(object) {
      return typeof object === 'boolean' || object instanceof Boolean;
    };

    var isFloat = function isFloat(value) {
      return Number(value) === value && value % 1 !== 0;
    };

    var isInt = function isInt(value) {
      return Number(value) === value && value % 1 === 0;
    };

    var isNumber = function isNumber(object) {
      return typeof object === 'number' || object instanceof Number;
    };

    var isString = function isString(object) {
      return typeof object === 'string' || object instanceof String;
    };

    var isUint = function isUint(value) {
      return Number(value) === value && value % 1 === 0 && value >= 0;
    };

    var contains = function contains(array, value) {
      return array instanceof Array ? array.indexOf(value) > -1 : false;
    };

    function initialize() {
      var elements = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      var ar = [];
      elements = elements > 0 ? Math.abs(elements) : 0;
      if (elements > 0) {
        for (var i = 0; i < elements; i++) {
          ar[i] = value;
        }
      }
      return ar;
    }

    function pierce(ar, index
    , flag) {
      index = index > 0 ? Math.abs(index) : 0;
      flag = Boolean(flag);
      var item = ar[index];
      ar.splice(index, 1);
      return flag ? ar : item;
    }

    function repeat(ar, count
    ) {
      var result = null;
      if (ar instanceof Array) {
        count = count > 0 ? count : 0;
        if (count > 0) {
          result = [];
          for (var i = 0; i < count; i++) {
            result = result.concat(ar);
          }
        } else {
          result = [].concat(ar);
        }
      }
      return result;
    }

    function rotate(ar) {
      var amount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
      if (ar instanceof Array && ar.length > 0) {
        amount %= ar.length;
        if (amount > 0) {
          ar.unshift.apply(ar, ar.splice(-amount, amount));
        } else if (amount < 0) {
          ar.push.apply(ar, ar.splice(0, -amount));
        }
      } else {
        ar = null;
      }
      return ar;
    }

    function shuffle(ar) {
      if (ar instanceof Array) {
        var item;
        var rdm;
        var tmp = [];
        var len = ar.length;
        var index = len - 1;
        for (var i = 0; i < len; i++) {
          rdm = Math.round(Math.random() * index);
          item = ar[rdm];
          ar.splice(rdm, 1);
          tmp[tmp.length] = item;
          index--;
        }
        while (--len > -1) {
          ar[len] = tmp[len];
        }
      } else {
        ar = null;
      }
      return ar;
    }

    function sortOn(ar, propName, options) {
      var sort = function sort(o1, o2) {
        var v1 = propName in o1 ? o1[propName] : '';
        var v2 = propName in o2 ? o2[propName] : '';
        switch (options) {
          case Array.CASEINSENSITIVE:
          case Array.CASEINSENSITIVE | Array.RETURNINDEXEDARRAY:
            {
              v1 = typeof v1 === "string" || v1 instanceof String ? v1.toLowerCase() : v1;
              v2 = typeof v2 === "string" || v2 instanceof String ? v2.toLowerCase() : v2;
              break;
            }
          case Array.NUMERIC:
          case Array.NUMERIC | Array.RETURNINDEXEDARRAY:
            {
              v1 = Number(v1);
              v2 = Number(v2);
              v1 = isNaN(v1) ? 0 : v1;
              v2 = isNaN(v2) ? 0 : v2;
              break;
            }
          case Array.DESCENDING:
          case Array.DESCENDING | Array.RETURNINDEXEDARRAY:
            {
              var _ref = [v2, v1];
              v1 = _ref[0];
              v2 = _ref[1];
              break;
            }
          case Array.CASEINSENSITIVE | Array.DESCENDING:
          case Array.CASEINSENSITIVE | Array.DESCENDING | Array.RETURNINDEXEDARRAY:
            {
              v1 = typeof v1 === "string" || v1 instanceof String ? v1.toLowerCase() : v1;
              v2 = typeof v2 === "string" || v2 instanceof String ? v2.toLowerCase() : v2;
              var _ref2 = [v2, v1];
              v1 = _ref2[0];
              v2 = _ref2[1];
              break;
            }
          case Array.NUMERIC | Array.DESCENDING:
          case Array.NUMERIC | Array.DESCENDING | Array.RETURNINDEXEDARRAY:
            {
              v1 = Number(v1);
              v2 = Number(v2);
              v1 = isNaN(v1) ? 0 : v1;
              v2 = isNaN(v2) ? 0 : v2;
              var _ref3 = [v2, v1];
              v1 = _ref3[0];
              v2 = _ref3[1];
              break;
            }
          case Array.UNIQUESORT:
            {
              if (v1 === v2) {
                return;
              }
              break;
            }
        }
        if (v1 < v2) {
          return -1;
        } else if (v1 > v2) {
          return 1;
        } else {
          return 0;
        }
      };
      switch (options) {
        case Array.RETURNINDEXEDARRAY:
        case Array.RETURNINDEXEDARRAY | Array.NUMERIC:
        case Array.RETURNINDEXEDARRAY | Array.CASEINSENSITIVE:
        case Array.RETURNINDEXEDARRAY | Array.NUMERIC | Array.DESCENDING:
        case Array.RETURNINDEXEDARRAY | Array.CASEINSENSITIVE | Array.DESCENDING:
          {
            var tmp = [].concat(ar);
            tmp.sort(sort);
            var result = [];
            var l = ar.length;
            for (var i = 0; i < l; i++) {
              result.push(tmp.indexOf(ar[i]));
            }
            return result;
          }
        default:
          {
            return ar.sort(sort);
          }
      }
    }
    Array.CASEINSENSITIVE = 1;
    Array.DESCENDING = 2;
    Array.NUMERIC = 16;
    Array.RETURNINDEXEDARRAY = 8;
    Array.UNIQUESORT = 4;

    function spliceInto(inserted, container, position, count) {
      inserted.unshift(position, count);
      try {
        container.splice.apply(container, inserted);
      } finally {
        inserted.splice(0, 2);
      }
    }

    function swap(ar) {
      var from = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var to = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var clone = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
      if (ar instanceof Array) {
        if (clone) {
          ar = [].concat(ar);
        }
        var value = ar[from];
        ar[from] = ar[to];
        ar[to] = value;
        return ar;
      }
      return null;
    }

    /**
     * The {@link core.arrays} package is a modular <b>JavaScript</b> library that provides extra <code>Array</code> methods.
     * @summary The {@link core.arrays} package is a modular <b>JavaScript</b> library that provides extra <code>Array</code> methods.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.arrays
     * @memberof core
     * @since 1.0.0
     */
    var arrays = {
      contains: contains,
      initialize: initialize,
      pierce: pierce,
      repeat: repeat,
      rotate: rotate,
      shuffle: shuffle,
      sortOn: sortOn,
      spliceInto: spliceInto,
      swap: swap
    };

    function compare(charA, charB) {
      var a = charA.charAt(0);
      var b = charB.charAt(0);
      if (caseValue(a) < caseValue(b)) {
        return -1;
      }
      if (caseValue(a) > caseValue(b)) {
        return 1;
      }
      if (a < b) {
        return -1;
      }
      if (a > b) {
        return 1;
      }
      return 0;
    }
    var caseValue = function caseValue(str) {
      return str.toLowerCase().valueOf() === str.valueOf() ? 0 : 1;
    };

    function isAlpha(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return "A" <= c && c <= "Z" || "a" <= c && c <= "z";
    }

    function isAlphaOrDigit(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return "A" <= c && c <= "Z" || "a" <= c && c <= "z" || "0" <= c && c <= "9";
    }

    function isASCII(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return c.charCodeAt(0) <= 255;
    }

    function isContained(c, charset) {
      var index = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      var l = charset.length;
      for (var i = 0; i < l; i++) {
        if (c === charset.charAt(i)) {
          return true;
        }
      }
      return false;
    }

    function isDigit(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return "0" <= c && c <= "9";
    }

    function isHexDigit(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return "0" <= c && c <= "9" || "A" <= c && c <= "F" || "a" <= c && c <= "f";
    }

    function isIdentifierStart(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return "A" <= c && c <= "Z" || "a" <= c && c <= "z" || c === "_" || c === "$";
    }

    var lineTerminators = ["\n",
    "\r",
    "\u2028",
    "\u2929"];

    function isLineTerminator(c) {
      var index
      = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      var l = lineTerminators.length;
      while (--l > -1) {
        if (c === lineTerminators[l]) {
          return true;
        }
      }
      return false;
    }

    function isLower(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return "a" <= c && c <= "z";
    }

    function isOctalDigit(c) {
      var index
      = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return "0" <= c && c <= "7";
    }

    var operators = ["*", "/", "%", "+", "-", "«", "»", ">", "<", "›", "&", "^", "|"];

    function isOperator(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return operators.indexOf(c) > -1;
    }

    var symbols = [" ",
    "!",
    "\"",
    "#",
    "$",
    "%",
    "&",
    "\'",
    "(",
    ")",
    "*",
    "+",
    ",",
    "-",
    ".",
    "/",
    ":",
    ";",
    "<",
    "=",
    ">",
    "?",
    "@",
    "[",
    "\\",
    "]",
    "^",
    "_",
    "`",
    "{",
    "|",
    "}",
    "~"];

    function isSymbol(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return symbols.indexOf(c) > -1;
    }

    function isUnicode(c) {
      var index
      = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return c.charCodeAt(0) > 255;
    }

    function isUpper(c) {
      var index
      = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return "A" <= c && c <= "Z";
    }

    var whiteSpaces = ["\t",
    "\n",
    "\x0B",
    "\f",
    "\r",
    " ",
    "\xA0",
    "\u1680",
    "\u180E",
    "\u2000",
    "\u2001",
    "\u2002",
    "\u2003",
    "\u2004",
    "\u2005",
    "\u2006",
    "\u2007",
    "\u2008",
    "\u2009",
    "\u200A",
    "\u200B",
    "\u2028",
    "\u2029",
    "\u202F",
    "\u205F",
    "\u3000"];

    function isWhiteSpace(c) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (index > 0) {
        c = c.charAt(index);
      }
      return whiteSpaces.indexOf(c) > -1;
    }

    /**
     * The {@link core.chars} package is a modular <b>JavaScript</b> library that provides extra <code>String</code> methods to validate and transform the basic characters.
     * <p>You can use this library for example to parse a string (JSON, csv, etc.).</p>
     * @summary The {@link core.chars} package is a modular <b>JavaScript</b> library that provides extra <code>String</code> methods to validate and transform a basic character.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.chars
     * @memberof core
     */
    var chars = {
      compare: compare,
      isAlpha: isAlpha,
      isAlphaOrDigit: isAlphaOrDigit,
      isASCII: isASCII,
      isContained: isContained,
      isDigit: isDigit,
      isHexDigit: isHexDigit,
      isIdentifierStart: isIdentifierStart,
      isLineTerminator: isLineTerminator,
      isLower: isLower,
      isOctalDigit: isOctalDigit,
      isOperator: isOperator,
      isSymbol: isSymbol,
      isUnicode: isUnicode,
      isUpper: isUpper,
      isWhiteSpace: isWhiteSpace,
      lineTerminators: lineTerminators,
      operators: operators,
      symbols: symbols,
      whiteSpaces: whiteSpaces
    };

    var distance = function distance(color1, color2) {
      return Math.pow((color1 >> 16 & 0xFF) - (color2 >> 16 & 0xFF), 2) + Math.pow((color1 >> 8 & 0xFF) - (color2 >> 8 & 0xFF), 2) + Math.pow((color1 & 0xFF) - (color2 & 0xFF), 2);
    };

    var equals = function equals(color1, color2) {
      var tolerance = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0.01;
      var dist = Math.pow((color1 >> 16 & 0xFF) - (color2 >> 16 & 0xFF), 2) + Math.pow((color1 >> 8 & 0xFF) - (color2 >> 8 & 0xFF), 2) + Math.pow((color1 & 0xFF) - (color2 & 0xFF), 2);
      return dist <= tolerance * (255 * 255 * 3) << 0;
    };

    var fade = function fade() {
      var from = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var to = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0xFFFFFF;
      var ratio = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      if (ratio <= 0) {
        return from;
      } else if (ratio >= 1) {
        return to;
      }
      var r = from >> 16;
      var g = from >> 8 & 0xFF;
      var b = from & 0xFF;
      r += ((to >> 16) - r) * ratio;
      g += ((to >> 8 & 0xFF) - g) * ratio;
      b += ((to & 0xFF) - b) * ratio;
      return r << 16 | g << 8 | b;
    };

    var isLittleEndian = function isLittleEndian() {
      var a = new ArrayBuffer(4);
      var b = new Uint8Array(a);
      var c = new Uint32Array(a);
      b[0] = 0xa1;
      b[1] = 0xb2;
      b[2] = 0xc3;
      b[3] = 0xd4;
      if (c[0] === 0xd4c3b2a1) {
        return true;
      }
      if (c[0] === 0xa1b2c3d4) {
        return false;
      } else {
        return null;
      }
    };
    var littleEndian = isLittleEndian();

    var max = 0xFF;
    var fromARGB = function fromARGB(a, r, g, b) {
      r = Math.min(Math.max(r, 0), max);
      g = Math.min(Math.max(g, 0), max);
      b = Math.min(Math.max(b, 0), max);
      a = isNaN(a) ? 0 : a;
      a = 0xFF * Math.max(Math.min(a, 1), 0);
      return littleEndian ? (a << 24 | b << 16 | g << 8 | r) >>> 0 : (r << 24 | g << 16 | b << 8 | a) >>> 0;
    };

    var getAlpha = function getAlpha(color) {
      return color >> 24 & 0xFF;
    };

    var getBlue = function getBlue(color) {
      return color & 0xFF;
    };

    var getGreen = function getGreen(color) {
      return color >> 8 & 0xFF;
    };

    var getRed = function getRed(color) {
      return color >> 16 & 0xFF;
    };

    function hex(value) {
      var upper = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
      var hex = value.toString(16);
      hex = hex.length % 2 === 0 ? hex : "0" + hex;
      return upper ? hex.toUpperCase() : hex;
    }

    var isUnique = function isUnique(color, colors) {
      var tolerance = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0.01;
      if (!(colors instanceof Array) || colors.length === 0) {
        return true;
      }
      tolerance = tolerance * (255 * 255 * 3) << 0;
      var cur;
      var distance;
      var len = colors.length;
      for (var i = 0; i < len; i++) {
        cur = colors[i];
        distance = Math.pow((color >> 16 & 0xFF) - (cur >> 16 & 0xFF), 2) + Math.pow((color >> 8 & 0xFF) - (cur >> 8 & 0xFF), 2) + Math.pow((color & 0xFF) - (cur & 0xFF), 2);
        if (distance <= tolerance) {
          return false;
        }
      }
      return true;
    };

    var toHex = function toHex(value) {
      var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '#';
      var upper = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
      if (!(Number(value) === value && value % 1 === 0 && value >= 0)) {
        throw new TypeError('toHex failed, the value parameter must be an upper integer.');
      }
      prefix = typeof prefix === 'string' || prefix instanceof String ? prefix : '#';
      upper = upper === true;
      var gb;
      var r = value >> 16;
      gb = value ^ r << 16;
      var g = gb >> 8;
      var b = gb ^ g << 8;
      return prefix + hex(r, upper) + hex(g, upper) + hex(b, upper);
    };

    var uniques = function uniques(colors) {
      var maximum = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0xFFFFFF;
      var tolerance = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0.01;
      var result = [];
      for (var i = 0; i < colors.length && result.length < maximum; i++) {
        if (isUnique(colors[i], result, tolerance)) {
          result.push(colors[i]);
        }
      }
      return result;
    };

    /**
     * The {@link core.colors} package is a modular <b>JavaScript</b> library that provides extra <b>rgb color</b> methods.
     * @summary The {@link core.colors} package is a modular <b>JavaScript</b> library that provides extra <b>rgb color</b> methods.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.colors
     * @memberof core
     */
    var colors = {
      distance: distance,
      equals: equals,
      fade: fade,
      fromARGB: fromARGB,
      getAlpha: getAlpha,
      getBlue: getBlue,
      getGreen: getGreen,
      getRed: getRed,
      hex: hex,
      isUnique: isUnique,
      toHex: toHex,
      uniques: uniques
    };

    var ONE_DAY_MS = 86400000;

    var after = function after(date1, date2) {
      if (!(date1 instanceof Date && date2 instanceof Date)) {
        throw new TypeError('after() failed, the passed-in date arguments must be valid Date objects.');
      }
      return date1.valueOf() > date2.valueOf();
    };

    var before = function before(date1, date2) {
      if (!(date1 instanceof Date && date2 instanceof Date)) {
        throw new TypeError('after() failed, the passed-in date arguments must be valid Date objects.');
      }
      return date1.valueOf() < date2.valueOf();
    };

    var monthDays = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    var daysInMonth = function daysInMonth() {
      var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      if (!(date instanceof Date)) {
        date = new Date();
      }
      var y = date.getFullYear();
      var m = date.getMonth();
      if ((y % 4 === 0 && y % 100 !== 0 || y % 400 === 0) && m === 1) {
        return 29;
      }
      return monthDays[m];
    };

    var leapYear = function leapYear(date) {
      var year;
      if (date instanceof Date) {
        year = date.getFullYear();
      } else if (date instanceof Number || typeof date === 'number') {
        year = date;
      } else {
        throw new TypeError('leapYear failed, the passed-in date argument must be a valid Date object or an integer representing the year to evaluates.');
      }
      return year % 4 === 0 && year % 100 !== 0 || year % 400 === 0;
    };

    var yesterday = function yesterday() {
      var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var d = date instanceof Date ? date : new Date();
      return new Date(d.valueOf() - ONE_DAY_MS);
    };

    /**
     * The {@link core.date} package is a modular <b>JavaScript</b> library that provides extra <code>Date</code> methods.
     * @summary The {@link core.date} package is a modular <b>JavaScript</b> library that provides extra <code>Date</code> methods.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.date
     * @memberof core
     */
    var date = {
      ONE_DAY_MS: ONE_DAY_MS,
      after: after,
      before: before,
      daysInMonth: daysInMonth,
      leapYear: leapYear,
      yesterday: yesterday
    };

    var isDOMElement = function isDOMElement(value) {
      if (!value) {
        return false;
      }
      try {
        return value instanceof Node;
      } catch (e) {
        return false;
      }
    };

    var isDOMObject = function isDOMObject(value) {
      if (!value) {
        return false;
      }
      try {
        return value instanceof EventTarget;
      } catch (e) {
        return false;
      }
    };

    function _typeof(obj) {
      if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
        _typeof = function (obj) {
          return typeof obj;
        };
      } else {
        _typeof = function (obj) {
          return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
        };
      }

      return _typeof(obj);
    }

    var isElement = function isElement(value) {
      if (!value) {
        return false;
      }
      try {
        return value instanceof Element;
      } catch (e) {
        return !!(value && (typeof obj === "undefined" ? "undefined" : _typeof(obj)) === "object" && 'nodeType' in value && value.nodeType === 1 && value.nodeName);
      }
    };

    var isHTMLElement = function isHTMLElement(value) {
      if (!value) {
        return false;
      }
      if ("HTMLElement" in window) {
        return value && value instanceof HTMLElement;
      }
      return !!(value && _typeof(value) === "object" && 'nodeType' in value && value.nodeType === 1 && value.nodeName);
    };

    var isListObject = function isListObject(value) {
      if (!value) {
        return false;
      }
      try {
        return value instanceof NodeList;
      } catch (e) {
        return false;
      }
    };

    var isSVGElement = function isSVGElement(value) {
      if (!value) {
        return false;
      }
      if ("SVGElement" in window) {
        return value && value instanceof SVGElement;
      }
      return !!(value && _typeof(value) === "object" && 'nodeType' in value && value.nodeType === 1 && value.nodeName && value.xmlbase);
    };

    /**
     * The {@link core.dom} package is a modular <b>JavaScript</b> library that provides extra <code>W3C DOM</code> methods.
     * @summary The {@link core.dom} package is a modular <b>JavaScript</b> library that provides extra <code>W3C DOM</code> methods.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.dom
     * @memberof core
     */
    var dom = {
      isDOMElement: isDOMElement,
      isDOMObject: isDOMObject,
      isElement: isElement,
      isHTMLElement: isHTMLElement,
      isListObject: isListObject,
      isSVGElement: isSVGElement
    };

    var backIn = function backIn(t, b, c, d) {
      var s = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 1.70158;
      if (isNaN(s)) {
        s = 1.70158;
      }
      return c * (t /= d) * t * ((s + 1) * t - s) + b;
    };

    var backInOut = function backInOut(t, b, c, d) {
      var s = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 1.70158;
      if (isNaN(s)) {
        s = 1.70158;
      }
      if ((t /= d / 2) < 1) {
        return c / 2 * (t * t * (((s *= 1.525) + 1) * t - s)) + b;
      }
      return c / 2 * ((t -= 2) * t * (((s *= 1.525) + 1) * t + s) + 2) + b;
    };

    var backOut = function backOut(t, b, c, d) {
      var s = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 1.70158;
      if (isNaN(s)) {
        s = 1.70158;
      }
      return c * ((t = t / d - 1) * t * ((s + 1) * t + s) + 1) + b;
    };

    var bounceOut = function bounceOut(t, b, c, d) {
      if ((t /= d) < 1 / 2.75) {
        return c * (7.5625 * t * t) + b;
      } else if (t < 2 / 2.75) {
        return c * (7.5625 * (t -= 1.5 / 2.75) * t + 0.75) + b;
      } else if (t < 2.5 / 2.75) {
        return c * (7.5625 * (t -= 2.25 / 2.75) * t + 0.9375) + b;
      } else {
        return c * (7.5625 * (t -= 2.625 / 2.75) * t + 0.984375) + b;
      }
    };

    var bounceIn = function bounceIn(t, b, c, d) {
      return c - bounceOut(d - t, 0, c, d) + b;
    };

    var bounceInOut = function bounceInOut(t, b, c, d) {
      return t < d / 2 ? bounceIn(t * 2, 0, c, d) * 0.5 + b : bounceOut(t * 2 - d, 0, c, d) * 0.5 + c * 0.5 + b;
    };

    var circularIn = function circularIn(t, b, c, d) {
      return -c * (Math.sqrt(1 - (t /= d) * t) - 1) + b;
    };

    var circularInOut = function circularInOut(t, b, c, d) {
      if ((t /= d / 2) < 1) {
        return -c / 2 * (Math.sqrt(1 - t * t) - 1) + b;
      }
      return c / 2 * (Math.sqrt(1 - (t -= 2) * t) + 1) + b;
    };

    var circularOut = function circularOut(t, b, c, d) {
      return c * Math.sqrt(1 - (t = t / d - 1) * t) + b;
    };

    var cubicIn = function cubicIn(t, b, c, d) {
      return c * (t /= d) * t * t + b;
    };

    var cubicInOut = function cubicInOut(t, b, c, d) {
      if ((t /= d / 2) < 1) {
        return c / 2 * t * t * t + b;
      }
      return c / 2 * ((t -= 2) * t * t + 2) + b;
    };

    var cubicOut = function cubicOut(t, b, c, d) {
      return c * ((t = t / d - 1) * t * t + 1) + b;
    };

    var elasticIn = function elasticIn(t, b, c, d) {
      var a = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;
      var p = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0;
      var s;
      if (t === 0) {
        return b;
      }
      if ((t /= d) === 1) {
        return b + c;
      }
      if (!p) {
        p = d * 0.3;
      }
      if (!a || a < Math.abs(c)) {
        a = c;
        s = p / 4;
      } else {
        s = p / (2 * Math.PI) * Math.asin(c / a);
      }
      return -(a * Math.pow(2, 10 * (t -= 1)) * Math.sin((t * d - s) * (2 * Math.PI) / p)) + b;
    };

    var elasticInOut = function elasticInOut(t, b, c, d) {
      var a = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;
      var p = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0;
      var s;
      if (t === 0) {
        return b;
      }
      if ((t /= d / 2) === 2) {
        return b + c;
      }
      if (!p) {
        p = d * (0.3 * 1.5);
      }
      if (!a || a < Math.abs(c)) {
        a = c;
        s = p / 4;
      } else {
        s = p / (2 * Math.PI) * Math.asin(c / a);
      }
      if (t < 1) {
        return -0.5 * (a * Math.pow(2, 10 * (t -= 1)) * Math.sin((t * d - s) * (2 * Math.PI) / p)) + b;
      }
      return a * Math.pow(2, -10 * (t -= 1)) * Math.sin((t * d - s) * (2 * Math.PI) / p) * 0.5 + c + b;
    };

    var elasticOut = function elasticOut(t, b, c, d) {
      var a = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;
      var p = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0;
      var s;
      if (t === 0) {
        return b;
      }
      if ((t /= d) === 1) {
        return b + c;
      }
      if (!p) {
        p = d * 0.3;
      }
      if (!a || a < Math.abs(c)) {
        a = c;
        s = p / 4;
      } else {
        s = p / (2 * Math.PI) * Math.asin(c / a);
      }
      return a * Math.pow(2, -10 * t) * Math.sin((t * d - s) * (2 * Math.PI) / p) + c + b;
    };

    var expoIn = function expoIn(t, b, c, d) {
      return t === 0 ? b : c * Math.pow(2, 10 * (t / d - 1)) + b;
    };

    var expoInOut = function expoInOut(t, b, c, d) {
      if (t === 0) {
        return b;
      }
      if (t === d) {
        return b + c;
      }
      if ((t /= d / 2) < 1) {
        return c / 2 * Math.pow(2, 10 * (t - 1)) + b;
      }
      return c / 2 * (-Math.pow(2, -10 * --t) + 2) + b;
    };

    var expoOut = function expoOut(t, b, c, d) {
      return t === d ? b + c : c * (-Math.pow(2, -10 * t / d) + 1) + b;
    };

    var linear = function linear(t, b, c, d) {
      return c * t / d + b;
    };

    var quarticIn = function quarticIn(t, b, c, d) {
      return c * (t /= d) * t * t * t + b;
    };

    var quarticInOut = function quarticInOut(t, b, c, d) {
      if ((t /= d / 2) < 1) {
        return c / 2 * t * t * t * t + b;
      }
      return -c / 2 * ((t -= 2) * t * t * t - 2) + b;
    };

    var quarticOut = function quarticOut(t, b, c, d) {
      return -c * ((t = t / d - 1) * t * t * t - 1) + b;
    };

    var quinticIn = function quinticIn(t, b, c, d) {
      return c * (t /= d) * t * t * t * t + b;
    };

    var quinticInOut = function quinticInOut(t, b, c, d) {
      if ((t /= d / 2) < 1) {
        return c / 2 * t * t * t * t * t + b;
      }
      return c / 2 * ((t -= 2) * t * t * t * t + 2) + b;
    };

    var quinticOut = function quinticOut(t, b, c, d) {
      return c * ((t = t / d - 1) * t * t * t * t + 1) + b;
    };

    var regularIn = function regularIn(t, b, c, d) {
      return c * (t /= d) * t + b;
    };

    var regularInOut = function regularInOut(t, b, c, d) {
      if ((t /= d / 2) < 1) {
        return c / 2 * t * t + b;
      }
      return -c / 2 * (--t * (t - 2) - 1) + b;
    };

    var regularOut = function regularOut(t, b, c, d) {
      return -c * (t /= d) * (t - 2) + b;
    };

    var sineIn = function sineIn(t, b, c, d) {
      return -c * Math.cos(t / d * (Math.PI / 2)) + c + b;
    };

    var sineInOut = function sineInOut(t, b, c, d) {
      return -c / 2 * (Math.cos(Math.PI * t / d) - 1) + b;
    };

    var sineOut = function sineOut(t, b, c, d) {
      return c * Math.sin(t / d * (Math.PI / 2)) + b;
    };

    /**
     * The {@link system.transitions} package use the {@link core.easings} library who contains all the easing functions to create the specific <b>tweening</b> effects.
     * <p>These easings functions provide different flavors of math-based motion under a consistent API.</p>
     *
     * |  easing   |                         description                         |  in  | out  | inout  |
     * |:--------: |:----------------------------------------------------------: |:---: |:---: |:-----: |
     * |  linear   | simple linear tweening : no easing, no acceleration         |  -   |  -   |   -    |
     * |   back    | back easing : overshooting cubic easing: (s+1)*t^3 - s*t^2  | yes  | yes  |  yes   |
     * |  bounce   | bounce easing : exponentially decaying parabolic bounce     | yes  | yes  |  yes   |
     * | circular  | circular easing : sqrt(1-t^2)                               | yes  | yes  |  yes   |
     * |   cubic   | cubic easing : t^3                                          | yes  | yes  |  yes   |
     * |  elastic  | elastic easing : exponentially decaying sine wave           | yes  | yes  |  yes   |
     * |   expo    | exponential easing : 2^t                                    | yes  | yes  |  yes   |
     * |   quad    | quadratic easing : t^2                                      | yes  | yes  | yes    |
     * |  quartic  | quartic easing : t^4                                        | yes  | yes  |  yes   |
     * |  quintic  | quintic easing : t^5                                        | yes  | yes  |  yes   |
     * |  regular  | regular easing                                              | yes  | yes  |  yes   |
     * |   sine    | sinusoidal easing : sin(t)                                  | yes  | yes  |  yes   |
     * @summary The {@link core.easings} library contains all the easing functions to create the specific tweening effects.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @namespace core.easings
     * @memberof core
     * @since 1.0.0
     */
    var easings = {
      backIn: backIn,
      backInOut: backInOut,
      backOut: backOut,
      bounceIn: bounceIn,
      bounceInOut: bounceInOut,
      bounceOut: bounceOut,
      circularIn: circularIn,
      circularInOut: circularInOut,
      circularOut: circularOut,
      cubicIn: cubicIn,
      cubicInOut: cubicInOut,
      cubicOut: cubicOut,
      elasticIn: elasticIn,
      elasticInOut: elasticInOut,
      elasticOut: elasticOut,
      expoIn: expoIn,
      expoInOut: expoInOut,
      expoOut: expoOut,
      linear: linear,
      quarticIn: quarticIn,
      quarticInOut: quarticInOut,
      quarticOut: quarticOut,
      quinticIn: quinticIn,
      quinticInOut: quinticInOut,
      quinticOut: quinticOut,
      regularIn: regularIn,
      regularInOut: regularInOut,
      regularOut: regularOut,
      sineIn: sineIn,
      sineInOut: sineInOut,
      sineOut: sineOut
    };

    var aop = function aop(func) {
      var begin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      var end = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      var scope = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;
      return function () {
        try {
          if (begin !== null && begin instanceof Function) {
            begin();
          }
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }
          return func.apply(scope, args);
        } finally {
          if (end !== null && end instanceof Function) {
            end();
          }
        }
      };
    };

    /**
     * The {@link core.functors} package is a modular <b>JavaScript</b> library that provides extra <code>Function</code> methods.
     * @summary The {@link core.functors} package is a modular <b>JavaScript</b> library that provides extra <code>Function</code> methods.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.functors
     * @memberof core
     */
    var functors = {
      aop: aop
    };

    var RAD2DEG = 180 / Math.PI;

    var acosD = function acosD(ratio) {
      return Math.acos(ratio) * RAD2DEG;
    };

    var acosHm = function acosHm(x) {
      return Math.log(x - Math.sqrt(x * x - 1));
    };

    var acosHp = function acosHp(x) {
      return Math.log(x + Math.sqrt(x * x - 1));
    };

    var angleOfLine = function angleOfLine(x1, y1, x2, y2) {
      return Math.atan2(y2 - y1, x2 - x1) * RAD2DEG;
    };

    var asinD = function asinD(ratio) {
      return Math.asin(ratio) * RAD2DEG;
    };

    var asinH = function asinH(x) {
      return Math.log(x + Math.sqrt(x * x + 1));
    };

    var atan2D = function atan2D(y, x) {
      return Math.atan2(y, x) * RAD2DEG;
    };

    var atanD = function atanD(angle) {
      return Math.atan(angle) * RAD2DEG;
    };

    var atanH = function atanH(x) {
      return Math.log((1 + x) / (1 - x)) / 2;
    };

    var DEG2RAD = Math.PI / 180;

    var bearing = function bearing(latitude1, longitude1, latitude2, longitude2) {
      latitude1 = latitude1 * DEG2RAD;
      latitude2 = latitude2 * DEG2RAD;
      var dLng = (longitude2 - longitude1) * DEG2RAD;
      var y = Math.sin(dLng) * Math.cos(latitude2);
      var x = Math.cos(latitude1) * Math.sin(latitude2) - Math.sin(latitude1) * Math.cos(latitude2) * Math.cos(dLng);
      return (Math.atan2(y, x) * RAD2DEG + 360) % 360;
    };

    var clamp = function clamp(value, min, max) {
      if (isNaN(value)) {
        return NaN;
      }
      if (isNaN(min)) {
        min = value;
      }
      if (isNaN(max)) {
        max = value;
      }
      return Math.max(Math.min(value, max), min);
    };

    var berp = function berp(start, end, amount) {
      if (start === end) {
        return start;
      }
      amount = clamp(amount, 0, 1);
      amount = (Math.sin(amount * Math.PI * (0.2 + 2.5 * amount * amount * amount)) * Math.pow(1 - amount, 2.2) + amount) * (1 + 1.2 * (1 - amount));
      return start + (end - start) * amount;
    };

    var bounce = function bounce(amount) {
      return Math.abs(Math.sin(6.28 * (amount + 1) * (amount + 1)) * (1 - amount));
    };

    var cartesianToPolar = function cartesianToPolar(vector) {
      var degrees = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      return {
        angle: Math.atan2(vector.y, vector.x) * (degrees ? RAD2DEG : 1),
        radius: Math.sqrt(vector.x * vector.x + vector.y * vector.y)
      };
    };

    var ceil = function ceil(n) {
      var floatCount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (isNaN(n)) {
        return NaN;
      }
      var r = 1;
      var i = -1;
      while (++i < floatCount) {
        r *= 10;
      }
      return Math.ceil(n * r) / r;
    };

    var clerp = function clerp(start, end, amount) {
      var max = 360;
      var half = 180;
      var diff = end - start;
      if (diff < -half) {
        return start + (max - start + end) * amount;
      } else if (diff > half) {
        return start - (max - end + start) * amount;
      } else {
        return start + (end - start) * amount;
      }
    };

    var cosD = function cosD(angle) {
      return Math.cos(angle * DEG2RAD);
    };

    var coserp = function coserp(start, end, amount) {
      if (start === end) {
        return start;
      }
      amount = 1 - Math.cos(amount * Math.PI * 0.5);
      return (1 - amount) * start + amount * end;
    };

    var cosH = function cosH(x) {
      return (Math.exp(x) + Math.exp(-x)) / 2;
    };

    var degreesToRadians = function degreesToRadians(angle) {
      return angle * DEG2RAD;
    };

    var distance$1 = function distance(x1, y1, x2, y2) {
      var dx = x2 - x1;
      var dy = y2 - y1;
      return Math.sqrt(dx * dx + dy * dy);
    };

    var distanceByObject = function distanceByObject(p1, p2) {
      var dx = p2.x - p1.x;
      var dy = p2.y - p1.y;
      return Math.sqrt(dx * dx + dy * dy);
    };

    var EARTH_RADIUS_IN_METERS = 6371000;

    var EPSILON = 0.000000001;

    var factorial = function factorial(value) {
      if (value === 0) {
        return 1;
      }
      var result = value;
      while (--value) {
        result *= value;
      }
      return result;
    };

    var fibonacci = function fibonacci(value) {
      var i = 1;
      var j = 0;
      for (var k = 1; k <= value; k++) {
        var _ref = [j, i + j];
        i = _ref[0];
        j = _ref[1];
      }
      return j;
    };

    var finalBearing = function finalBearing(latitude1, longitude1, latitude2, longitude2) {
      latitude1 = latitude1 * DEG2RAD;
      latitude2 = latitude2 * DEG2RAD;
      var dLng = (longitude2 - longitude1) * DEG2RAD;
      var y = Math.sin(dLng) * Math.cos(latitude2);
      var x = Math.cos(latitude1) * Math.sin(latitude2) - Math.sin(latitude1) * Math.cos(latitude2) * Math.cos(dLng);
      return (Math.atan2(y, x) * RAD2DEG + 180) % 360;
    };

    var fixAngle = function fixAngle(angle) {
      if (isNaN(angle)) {
        angle = 0;
      }
      angle %= 360;
      return angle < 0 ? angle + 360 : angle;
    };

    var floor = function floor(n) {
      var floatCount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (isNaN(n)) {
        return NaN;
      }
      var r = 1;
      var i = -1;
      while (++i < floatCount) {
        r *= 10;
      }
      return Math.floor(n * r) / r;
    };

    var gcd = function gcd(i1, i2) {
      if (i2 === 0) {
        return i1;
      } else if (i1 === i2) {
        return i1;
      } else {
        var t;
        while (i2 !== 0) {
          t = i2;
          i2 = i1 % i2;
          i1 = t;
        }
        return i1;
      }
    };

    var haversine = function haversine(latitude1, longitude1, latitude2, longitude2) {
      var radius = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : EARTH_RADIUS_IN_METERS;
      if (isNaN(radius)) {
        radius = EARTH_RADIUS_IN_METERS;
      }
      var dLat = (latitude2 - latitude1) * DEG2RAD;
      var dLng = (longitude2 - longitude1) * DEG2RAD;
      var a = Math.sin(dLat * 0.5) * Math.sin(dLat * 0.5) + Math.cos(latitude1 * DEG2RAD) * Math.cos(latitude2 * DEG2RAD) * Math.sin(dLng / 2) * Math.sin(dLng / 2);
      var c = Number((2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a)) * radius).toFixed(3));
      return c === c ? c : 0;
    };

    var hermite = function hermite(start, end, amount) {
      if (start === end) {
        return start;
      }
      amount = amount * amount * (3 - 2 * amount);
      return (1 - amount) * start + amount * end;
    };

    var hypothenuse = function hypothenuse(x, y) {
      return Math.sqrt(x * x + y * y);
    };

    var interpolate = function interpolate(value, min, max) {
      return min + (max - min) * value;
    };

    var isEven = function isEven(value) {
      return value % 2 === 0;
    };

    var isOdd = function isOdd(value) {
      return value % 2 !== 0;
    };

    var LAMBDA = 0.57721566490143;

    var lerp = function lerp(start, end, amount) {
      if (start === end) {
        return start;
      }
      return (1 - amount) * start + amount * end;
    };

    var log10 = function log10(value) {
      return Math.log(value) / Math.LN10;
    };

    var logN = function logN(value, base) {
      return Math.log(value) / Math.log(base);
    };

    var normalize = function normalize(value, minimum, maximum) {
      return (value - minimum) / (maximum - minimum);
    };

    var map = function map(value, min1, max1, min2, max2) {
      return interpolate(normalize(value, min1, max1), min2, max2);
    };

    var midPoint = function midPoint(latitude1, longitude1, latitude2, longitude2) {
      var dLng = (longitude2 - longitude1) * DEG2RAD;
      latitude1 = latitude1 * DEG2RAD;
      longitude1 = longitude1 * DEG2RAD;
      latitude2 = latitude2 * DEG2RAD;
      var bx = Math.cos(latitude2) * Math.cos(dLng);
      var by = Math.cos(latitude2) * Math.sin(dLng);
      var point = {
        x: Math.atan2(Math.sin(latitude1) + Math.sin(latitude2), Math.sqrt((Math.cos(latitude1) + bx) * (Math.cos(latitude1) + bx) + by * by)) * RAD2DEG,
        y: (longitude1 + Math.atan2(by, Math.cos(latitude1) + bx)) * RAD2DEG
      };
      return point;
    };

    var MILE_TO_METER = 1609;

    var modulo = function modulo(a, b) {
      var r = a % b;
      return r * b < 0 ? r + b : r;
    };

    var nearlyEquals = function nearlyEquals(value1, value2) {
      var tolerance = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0.000001;
      if (isNaN(tolerance)) {
        tolerance = 0.000001;
      } else if (tolerance < 0) {
        tolerance = 0;
      }
      return Math.abs(value1 - value2) <= tolerance;
    };

    var percentage = function percentage(value, maximum) {
      var p = value / maximum * 100;
      return isNaN(p) || !isFinite(p) ? NaN : p;
    };

    var PHI = 1.61803398874989;

    var PI2 = Math.PI * 2;

    var polarToCartesian = function polarToCartesian(vector, degrees) {
      var angle = vector.angle;
      var radius = vector.radius;
      if (degrees) {
        angle *= DEG2RAD;
      }
      return {
        x: radius * Math.cos(angle),
        y: radius * Math.sin(angle)
      };
    };

    var replaceNaN = function replaceNaN(value) {
      var defaultValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      return isNaN(value) ? defaultValue : value;
    };

    var round = function round(n) {
      var floatCount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (isNaN(n)) {
        return NaN;
      }
      var r = 1;
      var i = -1;
      while (++i < floatCount) {
        r *= 10;
      }
      return Math.round(n * r) / r;
    };

    var sign = function sign(n) {
      if (isNaN(n)) {
        throw new TypeError("sign failed, the passed-in value not must be NaN.");
      }
      return n < 0 ? -1 : 1;
    };

    var sinD = function sinD(angle) {
      return Math.sin(angle * DEG2RAD);
    };

    var sinerp = function sinerp(start, end, amount) {
      if (start === end) {
        return start;
      }
      amount = Math.sin(amount * Math.PI * 0.5);
      return (1 - amount) * start + amount * end;
    };

    var sinH = function sinH(x) {
      return (Math.exp(x) - Math.exp(-x)) * 0.5;
    };

    var tanD = function tanD(angle) {
      return Math.tan(angle * DEG2RAD);
    };

    var tanH = function tanH(x) {
      return sinH(x) / cosH(x);
    };

    function vincenty(latitude1, longitude1, latitude2, longitude2)
    {
      var a = 6378137;
      var b = 6356752.3142;
      var f = 1 / 298.257223563;
      var L = (longitude2 - longitude1) * DEG2RAD;
      var U1 = Math.atan((1 - f) * Math.tan(latitude1 * DEG2RAD));
      var U2 = Math.atan((1 - f) * Math.tan(latitude2 * DEG2RAD));
      var sinU1 = Math.sin(U1),
          cosU1 = Math.cos(U1);
      var sinU2 = Math.sin(U2),
          cosU2 = Math.cos(U2);
      var lambda = L;
      var lambdaP = 2 * Math.PI;
      var iterLimit = 20;
      var cosLambda;
      var sinLambda;
      var cosSigma;
      var sinSigma;
      var sigma;
      var sinAlpha;
      var cosSqAlpha;
      var cos2SigmaM;
      var C;
      do {
        sinLambda = Math.sin(lambda);
        cosLambda = Math.cos(lambda);
        sinSigma = Math.sqrt(cosU2 * sinLambda * (cosU2 * sinLambda) + (cosU1 * sinU2 - sinU1 * cosU2 * cosLambda) * (cosU1 * sinU2 - sinU1 * cosU2 * cosLambda));
        if (sinSigma === 0) {
          return 0;
        }
        cosSigma = sinU1 * sinU2 + cosU1 * cosU2 * cosLambda;
        sigma = Math.atan2(sinSigma, cosSigma);
        sinAlpha = cosU1 * cosU2 * sinLambda / sinSigma;
        cosSqAlpha = 1 - sinAlpha * sinAlpha;
        cos2SigmaM = cosSigma - 2 * sinU1 * sinU2 / cosSqAlpha;
        if (isNaN(cos2SigmaM)) {
          cos2SigmaM = 0;
        }
        C = f / 16 * cosSqAlpha * (4 + f * (4 - 3 * cosSqAlpha));
        lambdaP = lambda;
        lambda = L + (1 - C) * f * sinAlpha * (sigma + C * sinSigma * (cos2SigmaM + C * cosSigma * (-1 + 2 * cos2SigmaM * cos2SigmaM)));
      } while (Math.abs(lambda - lambdaP) > 1e-12 && --iterLimit > 0);
      if (iterLimit === 0) {
        return NaN;
      }
      var uSq = cosSqAlpha * (a * a - b * b) / (b * b);
      var A = 1 + uSq / 16384 * (4096 + uSq * (-768 + uSq * (320 - 175 * uSq)));
      var B = uSq / 1024 * (256 + uSq * (-128 + uSq * (74 - 47 * uSq)));
      var deltaSigma = B * sinSigma * (cos2SigmaM + B / 4 * (cosSigma * (-1 + 2 * cos2SigmaM * cos2SigmaM) - B / 6 * cos2SigmaM * (-3 + 4 * sinSigma * sinSigma) * (-3 + 4 * cos2SigmaM * cos2SigmaM)));
      var s = b * A * (sigma - deltaSigma);
      s = Number(s.toFixed(3));
      return s;
    }

    function wrap(angle) {
      var min = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var max = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 360;
      var range = max - min;
      if (range <= 0) {
        return 0;
      }
      var result = (angle - min) % range;
      if (result < 0) {
        result += range;
      }
      return result + min;
    }

    /**
     * The {@link core.maths} package is a modular <b>JavaScript</b> library that provides extra <code>mathematics</code> methods and implementations.
     * @summary The {@link core.maths} package is a modular <b>JavaScript</b> library that provides extra <code>mathematics</code> methods and implementations.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.maths
     * @memberof core
     */
    var maths = {
      acosD: acosD,
      acosHm: acosHm,
      acosHp: acosHp,
      angleOfLine: angleOfLine,
      asinD: asinD,
      asinH: asinH,
      atan2D: atan2D,
      atanD: atanD,
      atanH: atanH,
      bearing: bearing,
      berp: berp,
      bounce: bounce,
      cartesianToPolar: cartesianToPolar,
      ceil: ceil,
      clamp: clamp,
      clerp: clerp,
      cosD: cosD,
      coserp: coserp,
      cosH: cosH,
      DEG2RAD: DEG2RAD,
      degreesToRadians: degreesToRadians,
      distance: distance$1,
      distanceByObject: distanceByObject,
      EARTH_RADIUS_IN_METERS: EARTH_RADIUS_IN_METERS,
      EPSILON: EPSILON,
      factorial: factorial,
      fibonacci: fibonacci,
      finalBearing: finalBearing,
      fixAngle: fixAngle,
      floor: floor,
      gcd: gcd,
      haversine: haversine,
      hermite: hermite,
      hypothenuse: hypothenuse,
      interpolate: interpolate,
      isEven: isEven,
      isOdd: isOdd,
      LAMBDA: LAMBDA,
      lerp: lerp,
      littleEndian: littleEndian,
      log10: log10,
      logN: logN,
      map: map,
      midPoint: midPoint,
      MILE_TO_METER: MILE_TO_METER,
      modulo: modulo,
      nearlyEquals: nearlyEquals,
      normalize: normalize,
      percentage: percentage,
      PHI: PHI,
      PI2: PI2,
      polarToCartesian: polarToCartesian,
      RAD2DEG: RAD2DEG,
      replaceNaN: replaceNaN,
      round: round,
      sign: sign,
      sinD: sinD,
      sinerp: sinerp,
      sinH: sinH,
      tanD: tanD,
      tanH: tanH,
      vincenty: vincenty,
      wrap: wrap
    };

    var clip = function clip(value, min, max) {
      if (value <= min) {
        return min;
      } else if (value >= max) {
        return max;
      }
      return value;
    };

    var toInt = function toInt(num) {
      return num - num % 1;
    };

    var toNumber = function toNumber(value) {
      var defaultValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      if (value === null || value === undefined) {
        return defaultValue;
      }
      if (typeof value === 'number') {
        return value;
      }
      return Number(value);
    };

    var toUint = function toUint(num) {
      num -= num % 1;
      return num < 0 ? -num : num;
    };

    /**
     * The {@link core.numbers} package is a modular <b>JavaScript</b> library that provides extra <code>Number</code> methods and implementations.
     * @summary The {@link core.numbers} package is a modular <b>JavaScript</b> library that provides extra <code>Number</code> methods and implementations.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.numbers
     * @memberof core
     */
    var numbers = {
      clip: clip,
      toInt: toInt,
      toNumber: toNumber,
      toUint: toUint,
      toUnicodeNotation: toUnicodeNotation
    };

    function forEach(object, callback) {
      var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      var breaker = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;
      if (!object) {
        return;
      }
      if ("forEach" in object && object.forEach instanceof Function) {
        object.forEach(callback, context);
      } else {
        for (var key in object) {
          if (key in object) {
            if (breaker !== null) {
              if (callback.call(context, object[key], key, object) === breaker) {
                return;
              }
            } else {
              callback.call(context, object[key], key, object);
            }
          }
        }
      }
    }

    function fuse(src, srcPos, dest, destPos, length) {
      if (!src) {
        throw new ReferenceError("fuse failed, if either src is null.");
      }
      if (!dest) {
        dest = src;
      }
      if (destPos < 0) {
        destPos = dest.length;
      }
      while (length > 0) {
        dest[destPos] = src[srcPos];
        srcPos++;
        destPos++;
        length--;
      }
    }

    function members(o) {
      var byValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      byValue = Boolean(byValue === true);
      var members = [];
      if (byValue) {
        for (var prop in o) {
          if (o.hasOwnProperty(prop)) {
            members.push(o[prop]);
          }
        }
      } else {
        for (var member in o) {
          if (o.hasOwnProperty(member)) {
            members.push(member);
          }
        }
      }
      return members.length > 0 ? members : null;
    }

    function merge(target, source) {
      var overwrite = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
      if (overwrite === null || overwrite === undefined) {
        overwrite = true;
      }
      if (source === null || source === undefined) {
        source = {};
      }
      for (var prop in source) {
        if (!(prop in target) || overwrite) {
          target[prop] = source[prop];
        }
      }
      return target;
    }

    /**
     * The {@link core.objects} package is a modular <b>JavaScript</b> library that provides extra <code>Object</code> methods and implementations.
     * @summary The {@link core.objects} package is a modular <b>JavaScript</b> library that provides extra <code>Object</code> methods and implementations.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.objects
     * @memberof core
     */
    var objects = {
      forEach: forEach,
      fuse: fuse,
      members: members,
      merge: merge
    };

    var generateUUID = function generateUUID() {
      var d = new Date().getTime();
      return pattern.replace(/[xy]/g, function (c) {
        var r = (d + Math.random() * 16) % 16 | 0;
        d = Math.floor(d / 16);
        return (c === 'x' ? r : r & 0x3 | 0x8).toString(16);
      });
    };
    var pattern = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx';

    /**
     * The {@link core.objects} package is a modular <b>JavaScript</b> library that provides extra methods to generates a random number.
     * @summary The {@link core.objects} package is a modular <b>JavaScript</b> library that provides extra methods to generates a random number.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.random
     * @memberof core
     */
    var random = {
      generateUUID: generateUUID
    };

    function getDefinitionByName(name) {
      var domain = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      if (name instanceof String || typeof name === 'string') {
        name = name.split('.');
        if (name.length > 0) {
          try {
            var o = domain || global;
            name.forEach(function (element) {
              if (o.hasOwnProperty(element)) {
                o = o[element];
              } else {
                return undefined;
              }
            });
            return o;
          } catch (e) {
          }
        }
      }
      return undefined;
    }

    function invoke(c) {
      var a = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      if (!(c instanceof Function)) {
        return null;
      }
      if (a === null || !(a instanceof Array) || a.length === 0) {
        return new c();
      }
      switch (a.length) {
        case 0:
          return new c();
        case 1:
          return new c(a[0]);
        case 2:
          return new c(a[0], a[1]);
        case 3:
          return new c(a[0], a[1], a[2]);
        case 4:
          return new c(a[0], a[1], a[2], a[3]);
        case 5:
          return new c(a[0], a[1], a[2], a[3], a[4]);
        case 6:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5]);
        case 7:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6]);
        case 8:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7]);
        case 9:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8]);
        case 10:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9]);
        case 11:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10]);
        case 12:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11]);
        case 13:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12]);
        case 14:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13]);
        case 15:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14]);
        case 16:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15]);
        case 17:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16]);
        case 18:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17]);
        case 19:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18]);
        case 20:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19]);
        case 21:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20]);
        case 22:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21]);
        case 23:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22]);
        case 24:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22], a[23]);
        case 25:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22], a[23], a[24]);
        case 26:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22], a[23], a[24], a[25]);
        case 27:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22], a[23], a[24], a[25], a[26]);
        case 28:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27]);
        case 29:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27], a[28]);
        case 30:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27], a[28], a[29]);
        case 31:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27], a[28], a[29], a[30]);
        case 32:
          return new c(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15], a[16], a[17], a[18], a[19], a[20], a[21], a[22], a[23], a[24], a[25], a[26], a[27], a[28], a[29], a[30], a[31]);
        default:
          return null;
      }
    }

    /**
     * The {@link core.reflect} package is a modular <b>JavaScript</b> library that provides extra methods to to obtain information about loaded objects or generate it.
     * @summary The {@link core.reflect} package is a modular <b>JavaScript</b> library that provides extra methods to to obtain information about loaded objects or generate it.
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @namespace core.reflect
     * @memberof core
     */
    var reflect = {
      getDefinitionByName: getDefinitionByName,
      invoke: invoke
    };

    function between(source, left, right) {
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      if (!(left instanceof String || typeof left === 'string') || left === "") {
        return source;
      }
      var start = source.indexOf(left);
      var end = source.indexOf(right, start + left.length);
      if (end < 0 || !(right instanceof String || typeof right === 'string') || right === "") {
        return source.substring(start + left.length);
      }
      return source.slice(start + left.length, end);
    }

    function camelCase(source) {
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      return source.replace(/-\D/g, function (match) {
        return match.charAt(1).toUpperCase();
      });
    }

    function capitalize(source) {
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      return source.replace(/\b[a-z]/g, function (match) {
        return match.toUpperCase();
      });
    }

    function center(source) {
      var size = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var separator = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : " ";
      if (source === null || !(source instanceof String || typeof source === 'string')) {
        return "";
      }
      if (separator === null || !(separator instanceof String || typeof separator === 'string')) {
        separator = " ";
      }
      var len = source.length;
      if (len <= size) {
        len = size - len;
        var remain = len % 2 === 0 ? "" : separator;
        var pad = "";
        var count = Math.floor(len / 2);
        if (count > 0) {
          for (var i = 0; i < count; i++) {
            pad = pad.concat(separator);
          }
        } else {
          pad = separator;
        }
        return pad + source + pad + remain;
      } else {
        return source;
      }
    }

    function trim(source) {
      var chars = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      if (!chars || !(chars instanceof Array)) {
        chars = whiteSpaces;
      }
      var i;
      var l;
      l = source.length;
      for (i = 0; i < l && chars.indexOf(source.charAt(i)) > -1; i++) {}
      source = source.substring(i);
      for (i = source.length - 1; i >= 0 && chars.indexOf(source.charAt(i)) > -1; i--) {}
      source = source.substring(0, i + 1);
      return source;
    }

    function clean(source) {
      if (source === null || !(source instanceof String || typeof source === 'string')) {
        return "";
      }
      return trim(source.replace(/\s+/g, ' '));
    }

    function compare$1(str1, str2) {
      var strict = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      if (!(typeof str1 === 'string' || str1 instanceof String)) {
        throw new TypeError('Bad arguments, the compare function failed, the first argument must be a string value.');
      }
      if (!(typeof str2 === 'string' || str2 instanceof String)) {
        throw new TypeError('Bad arguments, the compare function failed, the second argument must be a string value.');
      }
      if (str1 === str2) {
        return 0;
      }
      strict = Boolean(strict);
      if (!strict) {
        str1 = str1.toLowerCase();
        str2 = str2.toLowerCase();
      }
      if (str1.length === str2.length) {
        var local = str1.localeCompare(str2);
        if (local === 0) {
          return 0;
        } else if (local < 0) {
          return -1;
        }
        return 1;
      } else if (str1.length > str2.length) {
        return 1;
      } else {
        return -1;
      }
    }

    var deburr = function deburr(source) {
      return source && source.replace(reLatin, deburrLetter).replace(reComboMark, '');
    };
    var reLatin = /[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g;
    var rsComboMarksRange = "\\u0300-\\u036f\\ufe20-\\ufe23",
        rsComboSymbolsRange = "\\u20d0-\\u20f0";
    var rsCombo = '[' + rsComboMarksRange + rsComboSymbolsRange + ']';
    var reComboMark = RegExp(rsCombo, 'g');
    var deburredLetters = {
      '\xc0': 'A',
      '\xc1': 'A',
      '\xc2': 'A',
      '\xc3': 'A',
      '\xc4': 'A',
      '\xc5': 'A',
      '\xe0': 'a',
      '\xe1': 'a',
      '\xe2': 'a',
      '\xe3': 'a',
      '\xe4': 'a',
      '\xe5': 'a',
      '\xc7': 'C',
      '\xe7': 'c',
      '\xd0': 'D',
      '\xf0': 'd',
      '\xc8': 'E',
      '\xc9': 'E',
      '\xca': 'E',
      '\xcb': 'E',
      '\xe8': 'e',
      '\xe9': 'e',
      '\xea': 'e',
      '\xeb': 'e',
      '\xcc': 'I',
      '\xcd': 'I',
      '\xce': 'I',
      '\xcf': 'I',
      '\xec': 'i',
      '\xed': 'i',
      '\xee': 'i',
      '\xef': 'i',
      '\xd1': 'N',
      '\xf1': 'n',
      '\xd2': 'O',
      '\xd3': 'O',
      '\xd4': 'O',
      '\xd5': 'O',
      '\xd6': 'O',
      '\xd8': 'O',
      '\xf2': 'o',
      '\xf3': 'o',
      '\xf4': 'o',
      '\xf5': 'o',
      '\xf6': 'o',
      '\xf8': 'o',
      '\xd9': 'U',
      '\xda': 'U',
      '\xdb': 'U',
      '\xdc': 'U',
      '\xf9': 'u',
      '\xfa': 'u',
      '\xfb': 'u',
      '\xfc': 'u',
      '\xdd': 'Y',
      '\xfd': 'y',
      '\xff': 'y',
      '\xc6': 'Ae',
      '\xe6': 'ae',
      '\xde': 'Th',
      '\xfe': 'th',
      '\xdf': 'ss',
      "\u0100": 'A',
      "\u0102": 'A',
      "\u0104": 'A',
      "\u0101": 'a',
      "\u0103": 'a',
      "\u0105": 'a',
      "\u0106": 'C',
      "\u0108": 'C',
      "\u010A": 'C',
      "\u010C": 'C',
      "\u0107": 'c',
      "\u0109": 'c',
      "\u010B": 'c',
      "\u010D": 'c',
      "\u010E": 'D',
      "\u0110": 'D',
      "\u010F": 'd',
      "\u0111": 'd',
      "\u0112": 'E',
      "\u0114": 'E',
      "\u0116": 'E',
      "\u0118": 'E',
      "\u011A": 'E',
      "\u0113": 'e',
      "\u0115": 'e',
      "\u0117": 'e',
      "\u0119": 'e',
      "\u011B": 'e',
      "\u011C": 'G',
      "\u011E": 'G',
      "\u0120": 'G',
      "\u0122": 'G',
      "\u011D": 'g',
      "\u011F": 'g',
      "\u0121": 'g',
      "\u0123": 'g',
      "\u0124": 'H',
      "\u0126": 'H',
      "\u0125": 'h',
      "\u0127": 'h',
      "\u0128": 'I',
      "\u012A": 'I',
      "\u012C": 'I',
      "\u012E": 'I',
      "\u0130": 'I',
      "\u0129": 'i',
      "\u012B": 'i',
      "\u012D": 'i',
      "\u012F": 'i',
      "\u0131": 'i',
      "\u0134": 'J',
      "\u0135": 'j',
      "\u0136": 'K',
      "\u0137": 'k',
      "\u0138": 'k',
      "\u0139": 'L',
      "\u013B": 'L',
      "\u013D": 'L',
      "\u013F": 'L',
      "\u0141": 'L',
      "\u013A": 'l',
      "\u013C": 'l',
      "\u013E": 'l',
      "\u0140": 'l',
      "\u0142": 'l',
      "\u0143": 'N',
      "\u0145": 'N',
      "\u0147": 'N',
      "\u014A": 'N',
      "\u0144": 'n',
      "\u0146": 'n',
      "\u0148": 'n',
      "\u014B": 'n',
      "\u014C": 'O',
      "\u014E": 'O',
      "\u0150": 'O',
      "\u014D": 'o',
      "\u014F": 'o',
      "\u0151": 'o',
      "\u0154": 'R',
      "\u0156": 'R',
      "\u0158": 'R',
      "\u0155": 'r',
      "\u0157": 'r',
      "\u0159": 'r',
      "\u015A": 'S',
      "\u015C": 'S',
      "\u015E": 'S',
      "\u0160": 'S',
      "\u015B": 's',
      "\u015D": 's',
      "\u015F": 's',
      "\u0161": 's',
      "\u0162": 'T',
      "\u0164": 'T',
      "\u0166": 'T',
      "\u0163": 't',
      "\u0165": 't',
      "\u0167": 't',
      "\u0168": 'U',
      "\u016A": 'U',
      "\u016C": 'U',
      "\u016E": 'U',
      "\u0170": 'U',
      "\u0172": 'U',
      "\u0169": 'u',
      "\u016B": 'u',
      "\u016D": 'u',
      "\u016F": 'u',
      "\u0171": 'u',
      "\u0173": 'u',
      "\u0174": 'W',
      "\u0175": 'w',
      "\u0176": 'Y',
      "\u0177": 'y',
      "\u0178": 'Y',
      "\u0179": 'Z',
      "\u017B": 'Z',
      "\u017D": 'Z',
      "\u017A": 'z',
      "\u017C": 'z',
      "\u017E": 'z',
      "\u0132": 'IJ',
      "\u0133": 'ij',
      "\u0152": 'Oe',
      "\u0153": 'oe',
      "\u0149": "'n",
      "\u017F": 'ss'
    };
    var basePropertyOf = function basePropertyOf(object) {
      return function (key) {
        return object == null ? undefined : object[key];
      };
    };
    var deburrLetter = basePropertyOf(deburredLetters);

    var empty = function empty(value) {
      return (typeof value === 'string' || value instanceof String) && value === '';
    };

    function endsWith(source, value) {
      if (!(source instanceof String || typeof source === 'string') || !(value instanceof String || typeof value === 'string') || source.length < value.length) {
        return false;
      }
      if (value === "") {
        return true;
      }
      return source.lastIndexOf(value) === source.length - value.length;
    }

    function fastformat(pattern) {
      for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }
      if (pattern === null || !(pattern instanceof String || typeof pattern === 'string')) {
        return "";
      }
      if (args.length > 0) {
        args = [].concat.apply([], args);
        var len = args.length;
        for (var i = 0; i < len; i++) {
          pattern = pattern.replace(new RegExp("\\{" + i + "\\}", "g"), args[i]);
        }
      }
      return pattern;
    }

    function fastformatDate() {
      var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var separator = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '-';
      if (!(date instanceof Date)) {
        date = new Date();
      }
      var month = date.getMonth() + 1;
      var day = date.getDate();
      var exp = date.getFullYear() + separator;
      if (month < 10) {
        exp += "0";
      }
      exp += month + separator;
      if (day < 10) {
        exp += "0";
      }
      exp += day;
      return exp;
    }

    function pad(source) {
      var amount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var expression = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : " ";
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      var left = amount >= 0;
      var width = amount > 0 ? amount : -amount;
      if (width < source.length || width === 0) {
        return source;
      }
      if (expression === null) {
        expression = " ";
      } else if (expression.length > 1) {
        expression = expression.charAt(0);
      }
      while (source.length !== width) {
        if (left) {
          source = expression + source;
        } else {
          source += expression;
        }
      }
      return source;
    }

    function format(pattern) {
      for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }
      if (pattern === null || !(pattern instanceof String || typeof pattern === 'string')) {
        return "";
      }
      var formatted = pattern;
      var len = args.length;
      var words = {};
      if (len === 1 && args[0] instanceof Array) {
        args = args[0];
      } else if (args[0] instanceof Array) {
        var a = args[0];
        args.shift();
        args = a.concat(args);
      } else if (args[0] instanceof Object && String(args[0]) === "[object Object]") {
        words = args[0];
        if (len > 1) {
          args.shift();
        }
      }
      var search = new RegExp("{([a-z0-9,:\\-]*)}", "m");
      var result = search.exec(formatted);
      var part;
      var token;
      var c;
      var pos;
      var dirty = false;
      var padding = 0;
      var buffer = [];
      while (result !== null) {
        part = result[0];
        token = result[1];
        pos = token.indexOf(",");
        if (pos > 0) {
          padding = Number(token.substr(pos + 1));
          token = token.substring(0, pos);
        }
        c = token.charAt(0);
        if ("0" <= c && c <= "9") {
          formatted = formatted.replace(part, pad(String(args[token]), padding));
        } else if (token === "" || token.indexOf(":") > -1) {
          buffer.push(part);
          formatted = formatted.replace(new RegExp(part, "g"), "\uFFFC" + (buffer.length - 1));
          dirty = true;
        } else if ("a" <= c && c <= "z") {
          if (token in words || words.hasOwnProperty(token)) {
            formatted = formatted.replace(new RegExp(part, "g"), pad(String(words[token]), padding));
          }
        } else {
          throw new Error("core.strings.format failed, malformed token \"" + part + "\", can not start with \"" + c + "\"");
        }
        result = search.exec(formatted);
      }
      if (dirty) {
        var i;
        var bl = buffer.length;
        for (i = 0; i < bl; i++) {
          formatted = formatted.replace(new RegExp("\uFFFC" + i, "g"), buffer[i]);
        }
      }
      return formatted;
    }

    function hyphenate(source) {
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      return source.replace(/[A-Z]/g, function (match) {
        return '-' + match.charAt(0).toLowerCase();
      });
    }

    function indexOfAny(source, anyOf) {
      var startIndex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var count = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : -1;
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return -1;
      }
      if (!(anyOf instanceof Array)) {
        return -1;
      }
      startIndex = startIndex > 0 ? 0 : startIndex;
      count = count < 0 ? -1 : count;
      var l = source.length;
      var endIndex;
      if (count < 0 || count > l - startIndex) {
        endIndex = l - 1;
      } else {
        endIndex = startIndex + count - 1;
      }
      for (var i = startIndex; i <= endIndex; i++) {
        if (anyOf.indexOf(source[i]) > -1) {
          return i;
        }
      }
      return -1;
    }

    function insert(source) {
      var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var value = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "";
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      if (!(value instanceof String || typeof value === 'string') || value === "") {
        return source;
      }
      if (index < 0) {
        index = source.length - Math.abs(index) + 1;
      } else if (index === 0) {
        return value + source;
      } else if (index >= source.length) {
        return source + value;
      }
      return source.substr(0, index) + value + source.substr(index);
    }

    function lastIndexOfAny(source, anyOf) {
      var startIndex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : Infinity;
      var count = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : Infinity;
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return -1;
      }
      if (!(anyOf instanceof Array) || anyOf.length === 0) {
        return -1;
      }
      if (startIndex < 0) {
        return -1;
      } else if (isNaN(startIndex) || startIndex > source.length) {
        startIndex = source.length;
      }
      count = count > 0 ? count : 0;
      var endIndex = Math.max(startIndex - count, 0);
      source = source.slice(endIndex, startIndex);
      var len = anyOf.length;
      for (var i = 0; i < len; i++) {
        var index = source.lastIndexOf(anyOf[i], startIndex);
        if (index > -1) {
          return endIndex + index;
        }
      }
      return -1;
    }

    var notEmpty = function notEmpty(value) {
      return (typeof value === 'string' || value instanceof String) && value !== '';
    };

    function repeat$1(source) {
      var count = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      count = isNaN(count) ? 0 : count;
      if (count < 0) {
        throw new RangeError('repeat count must be non-negative');
      }
      if (count === Infinity) {
        throw new RangeError('repeat count must be less than infinity');
      }
      count = Math.floor(count);
      if (source.length * count >= 1 << 28) {
        throw new RangeError('repeat count must not overflow maximum string size');
      }
      if (count === 0) {
        return "";
      }
      var result = '';
      for (;;) {
        if ((count & 1) === 1) {
          result += source;
        }
        count >>>= 1;
        if (count === 0) {
          break;
        }
        source += source;
      }
      return result;
    }

    var rsAstralRange = "\\ud800-\\udfff";
    var rsComboMarksRange$1 = "\\u0300-\\u036f";
    var reComboHalfMarksRange = "\\ufe20-\\ufe2f";
    var rsComboSymbolsRange$1 = "\\u20d0-\\u20ff";
    var rsComboMarksExtendedRange = "\\u1ab0-\\u1aff";
    var rsComboMarksSupplementRange = "\\u1dc0-\\u1dff";
    var rsComboRange = rsComboMarksRange$1 + reComboHalfMarksRange + rsComboSymbolsRange$1 + rsComboMarksExtendedRange + rsComboMarksSupplementRange;
    var rsDingbatRange = "\\u2700-\\u27bf";
    var rsLowerRange = 'a-z\\xdf-\\xf6\\xf8-\\xff';
    var rsMathOpRange = '\\xac\\xb1\\xd7\\xf7';
    var rsNonCharRange = '\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf';
    var rsPunctuationRange = "\\u2000-\\u206f";
    var rsSpaceRange = " \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000";
    var rsUpperRange = 'A-Z\\xc0-\\xd6\\xd8-\\xde';
    var rsVarRange = "\\ufe0e\\ufe0f";
    var rsBreakRange = rsMathOpRange + rsNonCharRange + rsPunctuationRange + rsSpaceRange;
    var rsApos = "['\u2019]";
    var rsBreak = "[".concat(rsBreakRange, "]");
    var rsCombo$1 = "[".concat(rsComboRange, "]");
    var rsDigits = '\\d';
    var rsDingbat = "[".concat(rsDingbatRange, "]");
    var rsLower = "[".concat(rsLowerRange, "]");
    var rsMisc = "[^".concat(rsAstralRange).concat(rsBreakRange + rsDigits + rsDingbatRange + rsLowerRange + rsUpperRange, "]");
    var rsFitz = "\\ud83c[\\udffb-\\udfff]";
    var rsModifier = "(?:".concat(rsCombo$1, "|").concat(rsFitz, ")");
    var rsNonAstral = "[^".concat(rsAstralRange, "]");
    var rsRegional = "(?:\\ud83c[\\udde6-\\uddff]){2}";
    var rsSurrPair = "[\\ud800-\\udbff][\\udc00-\\udfff]";
    var rsUpper = "[".concat(rsUpperRange, "]");
    var rsZWJ = "\\u200d";
    var rsMiscLower = "(?:".concat(rsLower, "|").concat(rsMisc, ")");
    var rsMiscUpper = "(?:".concat(rsUpper, "|").concat(rsMisc, ")");
    var rsOptContrLower = "(?:".concat(rsApos, "(?:d|ll|m|re|s|t|ve))?");
    var rsOptContrUpper = "(?:".concat(rsApos, "(?:D|LL|M|RE|S|T|VE))?");
    var reOptMod = "".concat(rsModifier, "?");
    var rsOptVar = "[".concat(rsVarRange, "]?");
    var rsOptJoin = "(?:".concat(rsZWJ, "(?:").concat([rsNonAstral, rsRegional, rsSurrPair].join('|'), ")").concat(rsOptVar + reOptMod, ")*");
    var rsOrdLower = '\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])';
    var rsOrdUpper = '\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])';
    var rsSeq = rsOptVar + reOptMod + rsOptJoin;
    var rsEmoji = "(?:".concat([rsDingbat, rsRegional, rsSurrPair].join('|'), ")").concat(rsSeq);
    var reUnicodeWord = RegExp([rsUpper + '?' + rsLower + '+' + rsOptContrLower + '(?=' + [rsBreak, rsUpper, '$'].join('|') + ')', rsMiscUpper + '+' + rsOptContrUpper + '(?=' + [rsBreak, rsUpper + rsMiscLower, '$'].join('|') + ')', rsUpper + '?' + rsMiscLower + '+' + rsOptContrLower, rsUpper + '+' + rsOptContrUpper, rsOrdUpper, rsOrdLower, rsDigits, rsEmoji].join('|'), 'g');
    var unicodeWords = function unicodeWords(source) {
      return source.match(reUnicodeWord) || [];
    };

    var reAsciiWord = /[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g;
    var reHasUnicodeWord = /[a-z][A-Z]|[A-Z]{2,}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/;
    var asciiWords = function asciiWords(string) {
      return string.match(reAsciiWord) || [];
    };
    var hasUnicodeWord = function hasUnicodeWord(string) {
      return reHasUnicodeWord.test(string);
    };
    var words = function words(source) {
      var pattern = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var guard = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      pattern = guard ? undefined : pattern;
      if (pattern === undefined) {
        return hasUnicodeWord(source) ? unicodeWords(source) : asciiWords(source);
      }
      return source.match(pattern) || [];
    };

    var transform = function transform(result, word, index) {
      return result + (index ? '_' : '') + word.toLowerCase();
    };
    var rsApos$1 = "['\u2019]";
    var reApos = RegExp(rsApos$1, 'g');
    function snakeCase() {
      var source = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      if (!(source instanceof String || typeof source === 'string')) {
        return null;
      }
      return words(deburr(source).replace(reApos, '')).reduce(transform, '');
    }

    function startsWith(source, value) {
      if (!(source instanceof String || typeof source === 'string') || !(value instanceof String || typeof value === 'string') || source.length < value.length) {
        return false;
      }
      if (value === "") {
        return true;
      }
      if (source.charAt(0) !== value.charAt(0)) {
        return false;
      }
      return source.indexOf(value) === 0;
    }

    var toString = function toString(value) {
      var defaultValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      if (value === null || value === undefined) {
        return defaultValue;
      }
      if (value instanceof String || typeof value === 'string') {
        return value;
      }
      return String(value);
    };

    function trimEnd(source) {
      var chars = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      if (!chars || !(chars instanceof Array)) {
        chars = whiteSpaces;
      }
      var i;
      for (i = source.length - 1; i >= 0 && chars.indexOf(source.charAt(i)) > -1; i--) {}
      return source.substring(0, i + 1);
    }

    function trimStart(source) {
      var chars = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      if (!chars || !(chars instanceof Array)) {
        chars = whiteSpaces;
      }
      var i;
      var l = source.length;
      for (i = 0; i < l && chars.indexOf(source.charAt(i)) > -1; i++) {}
      return source.substring(i);
    }

    var truncate = function truncate(source) {
      var length = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var prune = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "...";
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      length = length > 0 ? length : 0;
      if (!(prune instanceof String || typeof prune === 'string')) {
        prune = '...';
      }
      if (source.length <= length) {
        return source;
      }
      var template = source.slice(0, length + 1).replace(/.(?=\W*\w*$)/g, function (c) {
        return c.toUpperCase() !== c.toLowerCase() ? 'A' : ' ';
      });
      if (template.slice(template.length - 2).match(/\w\w/)) {
        template = template.replace(/\s*\S+$/, '');
      } else {
        template = trimEnd(template.slice(0, template.length - 1));
      }
      return (template + prune).length > source.length ? source : source.slice(0, template.length) + prune;
    };

    function ucFirst(source) {
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      return source.charAt(0).toUpperCase() + source.substring(1);
    }

    function ucWords(source) {
      var separator = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : " ";
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return '';
      }
      var ar = source.split(separator);
      var l = ar.length;
      while (--l > -1) {
        ar[l] = ar[l].charAt(0).toUpperCase() + ar[l].substring(1);
      }
      return ar.join(separator);
    }

    var versionUUID = function versionUUID(uuid) {
      return uuid.charAt(14) | 0;
    };

    var pattern$1 = /^[0-9a-f]{8}-[0-9a-f]{4}-[1-4][0-9a-f]{3}-[0-9a-f]{4}-[0-9a-f]{12}$/i;
    var validateUUID = function validateUUID(source) {
      var version = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      if (!(source instanceof String || typeof source === 'string') || source === "") {
        return false;
      }
      source = source.toLowerCase();
      if (!pattern$1.test(source)) {
        return false;
      }
      if (!version) {
        version = versionUUID(source);
      } else if (versionUUID(source) !== version) {
        return false;
      }
      switch (version) {
        case 1:
        case 2:
          {
            return true;
          }
        case 3:
        case 4:
          {
            return ['8', '9', 'a', 'b'].indexOf(source.charAt(19)) !== -1;
          }
        default:
          {
            throw new Error('Invalid version provided.');
          }
      }
    };

    /**
     * The {@link core.strings} package is a modular <b>JavaScript</b> library that provides extra <code>String</code> methods.
     * @summary The {@link core.strings} package is a modular <b>JavaScript</b> library that provides extra <code>String</code> methods.
     * @namespace core.strings
     * @memberof core
     * @license {@link https://www.mozilla.org/en-US/MPL/2.0/|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
     * @author Marc Alcaraz <ekameleon@gmail.com>
     * @since 1.0.0
     */
    var strings = {
      between: between,
      camelCase: camelCase,
      capitalize: capitalize,
      center: center,
      clean: clean,
      compare: compare$1,
      deburr: deburr,
      empty: empty,
      endsWith: endsWith,
      fastformat: fastformat,
      fastformatDate: fastformatDate,
      format: format,
      hyphenate: hyphenate,
      indexOfAny: indexOfAny,
      insert: insert,
      lastIndexOfAny: lastIndexOfAny,
      notEmpty: notEmpty,
      pad: pad,
      repeat: repeat$1,
      snakeCase: snakeCase,
      startsWith: startsWith,
      toString: toString,
      trim: trim,
      trimEnd: trimEnd,
      trimStart: trimStart,
      truncate: truncate,
      ucFirst: ucFirst,
      ucWords: ucWords,
      validateUUID: validateUUID,
      versionUUID: versionUUID,
      words: words
    };

    var metas = Object.defineProperties({}, {
      name: {
        enumerable: true,
        value: 'vegas-js-core'
      },
      description: {
        enumerable: true,
        value: "A specialized set of basic functions utilities that are highly reusable without creating any dependencies : arrays, strings, chars, objects, numbers, maths, date, colors, etc."
      },
      version: {
        enumerable: true,
        value: '1.0.5'
      },
      license: {
        enumerable: true,
        value: "MPL-2.0 OR GPL-2.0+ OR LGPL-2.1+"
      },
      url: {
        enumerable: true,
        value: 'https://bitbucket.org/ekameleon/vegas-js-core'
      }
    });
    try {
      if (window) {
        window.addEventListener('load', function () {
          window.removeEventListener("load", load, false);
          sayHello(metas.name, metas.version, metas.url);
        }, false);
      }
    } catch (ignored) {}
    Object.defineProperty(exports, '__esModule', {
      value: true
    });
    exports.global = global;
    exports.dump = dump;
    exports.trace = trace;
    exports.performance = performance;
    exports.cancelAnimationFrame = requestAnimationFrame;
    exports.requestAnimationFrame = requestAnimationFrame;
    exports.isBoolean = isBoolean;
    exports.isFloat = isFloat;
    exports.isInt = isInt;
    exports.isNumber = isNumber;
    exports.isString = isString;
    exports.isUint = isUint;
    exports.arrays = arrays;
    exports.chars = chars;
    exports.colors = colors;
    exports.date = date;
    exports.dom = dom;
    exports.easings = easings;
    exports.functors = functors;
    exports.maths = maths;
    exports.numbers = numbers;
    exports.objects = objects;
    exports.random = random;
    exports.reflect = reflect;
    exports.strings = strings;
    exports.metas = metas;
    exports.sayHello = sayHello;
    exports.skipHello = skipHello;

})));
//# sourceMappingURL=vegas.core.js.map
