"use strict" ;

import global from './src/global.js'

import dump    from './src/dump'
import trace   from './src/trace'
import version from './src/version'

import cancelAnimationFrame  from './src/cancelAnimationFrame'
import requestAnimationFrame from './src/requestAnimationFrame'

import isBoolean from './src/isBoolean'
import isFloat   from './src/isFloat'
import isInt     from './src/isInt'
import isNil     from './src/isNil'
import isNumber  from './src/isNumber'
import isUint    from './src/isUint'
import isString  from './src/isString'

import arrays   from './src/arrays'
import chars    from './src/chars'
import colors   from './src/colors'
import date     from './src/date'
import dom      from './src/dom'
import easings  from './src/easings'
import functors from './src/functors'
import maths    from './src/maths'
import numbers  from './src/numbers'
import objects  from './src/objects'
import random   from './src/random'
import reflect  from './src/reflect'
import strings  from './src/strings'

/**
 * The {@link core} package is specialized in functions utilities that are highly reusable without creating any dependencies : arrays, strings, chars, objects, numbers, maths, date, colors, etc.
 * <p>You can consider a library as a set of functions organized into classes, here with a <strong>"core"</strong> library in some cases we organize the functions in the package definitions without assembling them into a class.</p>
 * <p>Those functions are allowed to reuse the builtin types (Object, Array, Date, etc.), the Javascript API classes and packages, but nothing else.</p>
 * @summary The {@link core} package is specialized in functions utilities that are highly reusable without creating any dependencies.
 * @license {@link https://www.mozilla.org/en-US/MPL/2.0/)|MPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/gpl-2.0.fr.html|GPL 2.0} / {@link https://www.gnu.org/licenses/old-licenses/lgpl-2.1.fr.html|LGPL 2.1}
 * @author Marc Alcaraz <ekameleon@gmail.com>
 * @namespace core
 * @version 1.0.3
 * @since 1.0.0
 */
const core =
{
    global ,
    dump ,
    trace ,
    version ,
    
    cancelAnimationFrame  ,
    requestAnimationFrame ,

    isBoolean ,
    isFloat   ,
    isInt     ,
    isNil     ,
    isNumber  ,
    isString  ,
    isUint    ,

    arrays   ,
    chars    ,
    colors   ,
    date     ,
    dom      ,
    easings  ,
    functors ,
    maths    ,
    numbers  ,
    objects  ,
    random   ,
    reflect  ,
    strings
} ;

export default core ;